/** MID2226 API. API functions for the 2226 and 2224 boards.
 * Copyright 2009-2021 Sensoray Company Inc.
 * Provided as is under the standard Sensoray license.
 * This functions encapsulate the DirectShow functionality of the 2226 AVStream driver
 * Note: No support will be given for compiling or modifying the source code of this DLL API.
 * The 2224 is a smaller version of the 2226 without the full functionality 
 *  (such as MPEG-decode functionality, pass-through video outputs)
 */

#ifndef _MID2226FUNC_H
#define _MID2226FUNC_H

#define MAX_DEVICES 32  // maximum number of 2226s in system

#include "mid2226types.h"

#define MID2226_CURRENT_DRIVER ((1 << 24) | (0 << 16) | (62 << 8) | (0 << 0))
#define MID2226_CURRENT_PROXY ((1 << 24) | (0 << 17) | (1 << 8) | (19 << 0))
#define MID2226_CURRENT_DLL ((1 << 24) | (0 << 16) | (15 << 8) | (0 << 0)) 


/** Opens the SDK/API
  * @return 0 on success
  */
MID2226_API HANDLE __stdcall S2226_OpenBoard(int idx);

/** Cleans up and shuts down the API gracefully
 *  @return 0 on success
 */
MID2226_API int __stdcall S2226_CloseBoard(HANDLE hdev);

// enumerate
MID2226_API int __stdcall S2226_Enumerate(int *count, DEVINFO2226 *devices);


/** Sets the video input source. 
 *  @param input is the enumerated input MID2226_INPUT enum
 *  @return 0 on success
 */
MID2226_API int __stdcall S2226_SetInput(MID2226_INPUT input, BOOL bUpdateNow, HANDLE hdev);

// bitrate in kbps (1000-17000kbps allowed)
MID2226_API int __stdcall S2226_SetBitrate(unsigned long bitrate, HANDLE hdev);


/** Gets the current video source
 *  @param pSource pointer to returned value of current input source
 *  @return 0 on success 
*/
MID2226_API int __stdcall S2226_GetInput(MID2226_INPUT *pSource, HANDLE hdev );

/** set the current video system.  If board is currently streaming 
 *  board will be stopped and restarted.  If board recording, recording will
 *  be stopped.
 *  @param vidsys is the video system type
 *  @return 0 on success
 */
MID2226_API int __stdcall S2226_SetVidSys(MID2226_VIDSYS vidsys, HANDLE hdev );

MID2226_API int __stdcall S2226_GetVidSys(MID2226_VIDSYS *vidsys, HANDLE hdev);

// bReducedClock == 1 : 59.94Hz NTSC (affects NTSC high definition 720p, 1080i only!)
// bReducedClock == 0 : default settings and 60Hz NTSC
MID2226_API int __stdcall S2226_SetReducedClock(BOOL bReducedClock, HANDLE hdev);

MID2226_API int __stdcall S2226_GetReducedClock(BOOL *bReducedClock, HANDLE hdev);


/** retrieves current SDK and file information
 *  @return 0 on success, negative otherwise 
 */
MID2226_API int __stdcall S2226_GetStatus(MID2226STATUS *pStatus, HANDLE hdev );
//unicode version
MID2226_API int __stdcall S2226_GetStatusW(MID2226STATUS_W *pStatus, HANDLE hdev );

/** 
 * Starts recording to file using filter graph previously set up.
 * @param filename is the video/mux filename to record to.  Use the full path and extension
 * @return 0 on success
 */
MID2226_API int __stdcall S2226_StartRecord(const char *filename, HANDLE hdev);
// unicode
MID2226_API int __stdcall S2226_StartRecordW(const wchar_t *filename, HANDLE hdev);




/////////////////////////////////////////////////
// General configuration options

/** set the brightness, contrast and saturation
*   @param param is the parameter to change(MID2226_LEVEL_CONTRAST, MID2226_LEVEL_BRIGHTNESS, MID2226_LEVEL_SATURATION, MID2226_LEVEL_HUE)
*   @param value is the value of the parameter
*    ( BRIGHTNESS: 0 to 255 default 128)
*    ( SATURATION: -127 to 127 default 64)
*    ( CONTRAST: -127 to 127 default 68 (negative values not recommended))
*    ( HUE: -127 to 127, default 0 (recommended to stay unchanged))
*   @return 0 on success
*/
MID2226_API int __stdcall S2226_SetLevel(int param, int value, HANDLE hdev );

/** get the brightness, contrast or saturation
  * @param param is the parameter to get(MID2226_LEVEL_CONTRAST, MID2226_LEVEL_BRIGHTNESS, MID2226_LEVEL_SATURATION, MID2226_LEVEL_HUE)
  * @param value is the returned value of the parameter
  * @return 0 on success
  */
MID2226_API int __stdcall S2226_GetLevel(int param, int *value, HANDLE hdev );


// see manual.  default gain is 0dB
// gain is from 0 to 118 in steps of +.5dB
// maximum gain is 59dB
MID2226_API int __stdcall S2226_SetAudioGain(int gainL, int gainR, HANDLE hdev);
MID2226_API int __stdcall S2226_GetAudioGain(int *gainL, int *gainR, HANDLE hdev);
// Audio can use a standard gain stage or automatic gain control AGC.
// AGC may be desirable when for example recording speech.
// The gains in the functions below are the maximum gain used for AGC
// If AGC is on, the gains in the S2226_SetAudioGain function are NOT used.
// If AGC is off, the audio gain is controlled by the S2226_SetAudioGain function
// default is AGC disabled.
MID2226_API int __stdcall S2226_SetAudioAgc(BOOL bOnL, BOOL bOnR, int gainL, int gainR, HANDLE hdev);
MID2226_API int __stdcall S2226_GetAudioAgc(BOOL *bOnL, BOOL *bOnR, int *gainL, int *gainR, HANDLE hdev);

// Set bBalanced to TRUE if using a balanced or differential input, default is FALSE
MID2226_API int __stdcall S2226_SetAudioBalanced(BOOL bBalanced, HANDLE hdev);
MID2226_API int __stdcall S2226_GetAudioBalanced(BOOL *bBalanced, HANDLE hdev);


// Audio DAC volume for playback/decode on 2226 board
// (Does not apply for bypass audio routes )
// 0 - 127  (steps of -0.5 dB,  0==0dB==maximum volume,  127 == -63.5dB)
MID2226_API int __stdcall S2226_SetAudioOutputVol(int val, HANDLE hdev);
MID2226_API int __stdcall S2226_GetAudioOutputVol(int *val, HANDLE hdev);

// extra gain from 0 to 9 (steps of 1dB) (default recommended setting 0) Mono audio output
MID2226_API int __stdcall S2226_SetAudioOutputMono(int extra_gain, BOOL mute, HANDLE hdev);
MID2226_API int __stdcall S2226_GetAudioOutputMono(int *extra_gain, BOOL *mute, HANDLE hdev);

// extra gain from 0 to 9 (steps of 1dB, default recommended setting 0) High Power audio output
MID2226_API int __stdcall S2226_SetAudioOutputHp(int extra_gain, BOOL mute, HANDLE hdev);
MID2226_API int __stdcall S2226_GetAudioOutputHp(int *extra_gain, BOOL *mute, HANDLE hdev);

// extra gain from 0 to 9 (steps of 1dB default recommended setting 0) Stereo audio output
MID2226_API int __stdcall S2226_SetAudioOutputStereo(int extra_gain, BOOL mute, HANDLE hdev);
MID2226_API int __stdcall S2226_GetAudioOutputStereo(int *extra_gain, BOOL *mute, HANDLE hdev);

// See manual:
// StartCallback starts a stream that only flows through the callback.
// This stream will be the primary MPEG encoded H.264 stream.
// S2226_StartCallback requires S2226_RegisterCallback
// After S2226_StopStream, callback is reset.
// Callbacks must be short and return quickly.
// If not, buffer the data and process it in another thread.
// Do not call any stream control functions in the callback
MID2226_API int __stdcall S2226_StartCallback(HANDLE hdev);
typedef int (__stdcall *cbfunc_t) ( BYTE *merged, long size, HANDLE hdev);
// New 8/2/2011
// S2226_RegisterCallback will register a callback for the other streaming functions.
// To turn off callbacks, call with an argument of NULL for the callback function.
// The callback is for the MPEG stream only.
// Callbacks must be short and return quickly.
// If not, buffer the data and process it in another thread.
// Do not call any stream control functions in the callback
MID2226_API int __stdcall S2226_RegisterCallback( cbfunc_t callback, HANDLE hdev);

// New 8/2/2011.  Register callback for preview or raw uncompressed data
// Note: This may affect performance (CPU usage) as the raw data may not be DMA'd 
//       directly to video memory.
// Callbacks must be short and return quickly.
// If not, buffer the data and process it in another thread.
// Do not call any stream control functions in the callback
MID2226_API int __stdcall S2226_RegisterCallbackRaw( cbfunc_t callback, HANDLE hdev);


// Decoder must know what source/video system/clock the stream was encoded with (see MID2226_DECODE_TYPE)
// Applies to 2226 only.  Not support on the 2224
MID2226_API int __stdcall S2226_StartDecode(const char *filename, MID2226_DECODE_TYPE dt, HANDLE hdev); 
MID2226_API int __stdcall S2226_StartDecodeW(const wchar_t *filename, MID2226_DECODE_TYPE dt, HANDLE hdev); 

// decode sub modes.  Only valid when decode is running
MID2226_API int __stdcall S2226_PauseDecode(HANDLE hdev);

// hold is the number of 1/2 fields to hold the frame.  
// Use 4(faster, but not full speed) to 255 (slower)
MID2226_API int __stdcall S2226_SlowDecode(int hold, HANDLE hdev);   // slow decode
MID2226_API int __stdcall S2226_NormalDecode(HANDLE hdev); // normal speed


MID2226_API int __stdcall S2226_SetPreviewType(MID2226_PREVIEWTYPE type, HANDLE hdev);

// 2226-Rev 2.0 feature only.
// size= MID2226_RAWPREVIEW_LARGE or MID2226_RAWPREVIEW_SMALL (default)
// must be set before preview is started
MID2226_API int __stdcall S2226_SetRawPreviewSize(int size, HANDLE hdev);

// use hwnd = NULL for pop-up window.
MID2226_API int __stdcall S2226_StartPreview(HWND hwnd, HANDLE hdev);
// video preview and record to file
// use hwnd = NULL for pop-up window
MID2226_API int __stdcall S2226_StartPreviewAndRecord(HWND hwnd, const char *filename, HANDLE hdev);
// unicode version
// use hwnd = NULL for pop-up window
MID2226_API int __stdcall S2226_StartPreviewAndRecordW(HWND hwnd, const wchar_t *filename, HANDLE hdev);


// Stops any streaming operation( started with StartDecode, StartRecord, StartCallback, etc..
MID2226_API int __stdcall S2226_StopStream(HANDLE hdev); 


// Note: Start raw preview allows viewing the raw preview while streaming
//       It can be used with S2226_StartRecord or S2226_StartCallback
//       It should not be used with the S2226_StartPreview functions.
//       Use S2226_StopRawPreview to stop the independent preview stream
//       All streams must be stopped before the input can be changed.
//       If the input was changed and S2226_StartRawPreview is still running,
//       then the other S2226_Start functions will fail until S2226_StopRawPreview is
//       called.
MID2226_API int __stdcall S2226_StartRawPreview(HWND hwnd, HANDLE hdev);
MID2226_API int __stdcall S2226_StopRawPreview(HANDLE hdev);

// audio route.  For debug only. 
// route = 0 (normal audio input)
// route = 1 (alternate audio input for Sensoray testing)
// route = 2 (normal audio input, bypassed, test)
// route = 3 (alternat audio input, bypassed, test)
MID2226_API int __stdcall S2226_SetAudioRoute(int route, HANDLE hdev);
MID2226_API int __stdcall S2226_GetAudioRoute(int *route, HANDLE hdev);

//////////////////////////////////////////////////////////////////////////
// Overlay
//////////////////////////////////////////////////////////////////////////

/** Overlay text structure
 *  @param text pointer to text
 *  @param fontfile full name/path to font file
 *  @param fontsize height of font in pixels
 *  @param txtcolor text color in RGB format, example RGB(255,255,255)
 *  @param backcolor Background color, zero or less means transparent. (use small value eg. RGB(0,0,1) for black)
 */
typedef struct {
    char text[256];
    char fontfile[256]; 
    int  fontsize;  
    int  txtcolor;
    int  backcolor; 
    char underline;
    char bold;
    char italic;
    char reserved[7];
} overlay_text_t;

///** Adds overlay text.
//  * If overlay text already exists at that x,y position, deletes windows before adding.
//  * Overlay active on regions defined by regionmask.
//  *
//  * If text contains embedded newline characters (\n = 10 dec), then each line
//  * of text will be created on in a new window, AtIndex sub-window position, directly below 
//  * the preceding line. Each sub-window position only consumes enough overlay memory
//  * needed to hold the individual line.
//  *
//  * If text contains embedded character 30 dec (entered programmatically or by holding 
//  * down Alt- and typing "030" on the numeric keypad), then each line following a char(30)
//  * will be on a new line. Multi-line text created this way will be created as one large
//  * graphic at one AtIndex location.
//  *
//  * @param xpos       -- start X position
//  * @param ypos       -- start y position
//  * @param ovltext   -- pointer to overlay text
//  * @param regionmask -- region mask (MID2226_REGION_MONITOR, MID2226_REGION_MPEG, MID2226_REGION_STILL)
//  * @param hdev of 2226
//  * @return 0 on success, -1 on too many regions, other negative value failure
//  */
MID2226_API int __stdcall S2226_OverlayText( int xpos, int ypos, overlay_text_t *ovltext, int regionmask, HANDLE hdev);

///** Adds/Update overlay text.  If overlay already exists at index WinIndex, deletes window before adding.
//  * Overlay active on regions defined by regionmask.
//  * Multi-lines created as defined in S2226_OverlayText.
//  * @param AtIndex    -- sub-window position to update/add text
//  * @param xpos       -- new X position
//  * @param ypos       -- new y position
//  * @param ovltext   -- pointer to overlay text
//  * @param regionmask -- region mask (MID2226_REGION_MONITOR, MID2226_REGION_MPEG, MID2226_REGION_STILL)
//  * @param hdev of 2226.
//  * @return 0 on success, -1 on too many regions, other negative value failure
//  */
MID2226_API int __stdcall S2226_OverlayTextIdx( int AtIndex, int xpos, int ypos, overlay_text_t *ovltext, int regionmask, HANDLE hdev);

/** Adds overlay image. If overlay already exists at that x,y position, deletes window before adding.
  * @param xpos       -- start X position
  * @param ypos       -- start y position
  * @param imageFile  -- pointer to overlay image(full path)  24bit BMP or PCX files supported only
  * @param regionmask -- output region mask 
  * @param hdev of 2226.
  * @return 0 on success, -1 on too many regions, other negative value on failure
  */

MID2226_API int __stdcall S2226_OverlayImage( int xpos, int ypos, char *imageFile, int regionmask, HANDLE hdev);
/** Adds/Update overlay image. If overlay already exists at index WinIndex, deletes window before adding.
  * @param AtIndex    -- sub-window position to update/add text
  * @param xpos       -- start X position
  * @param ypos       -- start y position
  * @param imageFile  -- pointer to overlay image(full path)  24bit BMP or PCX files supported only
  * @param regionmask -- output region mask 
  * @param hdev of 2226.
  * @return 0 on success, -1 on too many regions, other negative value on failure
  */
MID2226_API int __stdcall S2226_OverlayImageIdx( int AtIndex, int xpos, int ypos, char *imageFile, int regionmask, HANDLE hdev);


///** Get basic parameters of an overlay index
// * @param WinIndex   -- sub-window position to get parameters from
// * @param Type       -- Type: 1=Image 0=Text
// * @param Region     -- Overlay Region, MID2226_REGION_MONITOR | MID2226_REGION_MPEG | MID2226_REGION_STILL
// * @param Group      -- Text with common group number is kept together. (-1 indicates not used)
// * @param Xpos       -- start X position
// * @param Ypos       -- start y position
// * @param Value      -- pointer to text or image file path
// * @return 0 on success
// */
MID2226_API int __stdcall S2226_GetOverlayIdx(int WinIndex, int *Type, int *Region, int *Group, int *Xpos, int *Ypos, char **Value,HANDLE hdev);
///** Set the color used for the background regions
//  * @param red    -- 0 min, 255 max red
//  * @param green  -- 0 min, 255 max green
//  * @param blue   -- 0 min, 255 max blue
//  * @param hdev of 2226.
//  * @return 0 on success
//  */
MID2226_API int __stdcall S2226_OverlayBackgroundColor( int red, int green, int blue, HANDLE hdev);
///** Change position of an overlay sub-window
//  * Does not reload or download the new image
//  * @param WinIndex   -- index of sub-window to move
//  * @param xpos       -- new X position
//  * @param ypos       -- new y position
//  * @param hdev of 2226.
//  * @return 0 on success, -1 overlay does not exist yet.
//  */
MID2226_API int __stdcall S2226_MoveOverlay( int WinIndex, int xpos, int ypos, HANDLE hdev);
///** Delete an overlay at x,y location.
//  * Does not reload or download the new image
//  * @param xpos       -- X position of sub-window to delete
//  * @param ypos       -- y position of sub-window to delete
//  * @param hdev of 2226.
//  * @return 0 on success, -1 overlay does not exist yet, -2 if that sub-window already unused.
//  */
MID2226_API int __stdcall S2226_OverlayDelXY( int xpos, int ypos, HANDLE hdev);
///** Delete an overlay at at a given index.
//  * Does not reload or download the new image
//  * @param WinIndex   -- index of sub-window to delete
//  * @param hdev of 2226.
//  * @return 0 on success, -1 overlay does not exist yet.
//  */
MID2226_API int __stdcall S2226_OverlayDel(int WinIndex, HANDLE hdev);
/** Refresh or update the overlay to the hardware
  * @param hdev of 2226.
  * @return 0 on success
  */
MID2226_API int __stdcall S2226_UpdateOverlay( HANDLE hdev);
///** Clear all overlays  
//  * @param hdev of 2226.
//  * @return 0 on success
//  */

MID2226_API int __stdcall S2226_ClearOverlay( HANDLE hdev);
///** Clears all overlays for specified region. Use S2226_ClearOverlay to erase overlays.  This function
//  * only clears the overlay by region
//  * @param regionmask output region mask
//  * @param hdev of 2226.
//  * @return 0 on success
//  */
MID2226_API int __stdcall S2226_ClearOverlayRegion( int regionmask, HANDLE hdev);

///** Set all overlays for specified region.This function
//  * only sets regionmask for all overlays
//  * @param regionmask output region mask
//  * @param hdev of 2226.
//  * @return 0 on success
//  */
MID2226_API int __stdcall S2226_SetOverlayRegion( int regionmask, HANDLE hdev);

/** Adds/Update overlay image. If overlay already exists at index AtIndex, deletes window before adding.
  * @param image      -- pointer to overlay image. 24bit RGB data array.
  * @param xpos       -- start X position
  * @param ypos       -- start y position
  * @param xSize      -- Horizontal size of image in pixels. (3 bytes per pixel)
  * @param ySize      -- Verticle size of image in pixels.
  * @param AtIndex    -- sub-window position to update/add text
  * @param regionmask -- output region mask 
  * @param handle     -- Handle of 2226.
  * @return 0 on success, -1 on too many regions, other negative value on failure
  */
MID2226_API int __stdcall S2226_OverlayImageRaw(BYTE *image, int xPos, int yPos, int xSize, int ySize, int AtIndex, int regionmask, HANDLE handle);

/** Copy given image to region within overlay zero.
  * Assumes Overlay zero is loaded into current memory
  * Assumes Overlay zero has a width that is divisible by 8
  * Assumes all other overlays (1-7) are contained within the
  * boundaries of overlay zero
  *
  * @param ImageFile   - Full name/path to image file. 24-bit bmp and pcx files are supported. If NULL, use Raw 24-bit bitmap pointer Image
  * @param ImageRGBptr - Pointer to memory containing 24-bit RGB data Row 0 first
  * @param xPos        - Horizontal Destination pixel position from left of Bitmap-0 to copy to.
  * @param yPos        - Vertical Destination pixel position from top of Bitmap-0 to copy to.
  * @param xSize       - Size of horizontal line in pixels (NOTE: Each pixel is 3 bytes: LSB=Red,Green,MSB=Blue) Used with Image pointer only.
  * @param ySize       - Number of Lines. Used with Image pointer only.
  * @param Backgnd_ForegndN - Copy directly to currently displayed overlay image when 0. Otherwise copy to overlay's double buffered background.
  * @param handle      - Handle of 2226.
  *
  * Returns 0 on success, -1 on failure. 
  */
MID2226_API int __stdcall S2226_CopyBmpToOverlayZero(char *ImageFile, char *Image, int xPos, int yPos, int xSize, int ySize, int Backgnd_ForegndN, HANDLE handle);

//////////////////////////////////////////////////////////////////////////
// Snapshot
//////////////////////////////////////////////////////////////////////////


///** Set the interlaced field merging algorithm 
//  * @param merge_method is the method in mid2226const.h.
//  * @param unused (reserved for future use, value ignored)
//  * @param hdev of 2226.
//  * @return 0 on success
//  */
MID2226_API int __stdcall S2226_SetMergeMethod( MID2226_FIELDALG merge_method, void *unused, HANDLE hdev);


/** Takes a snapshot and saves to file in filename
  * @param szFilename is the fully qualified file + path( without extension)
  * @param filetype is the file type to save( see mid2226const.h: MID2226_FILE_JPEG and/or MID2226_FILE_BMP)
  * @param freezetime is the time, in milli-sec to freeze the image. (minimum value greater than the video frame rate is required)
  * @param wait  wait =1 will wait if operations pending(board locked), 
  *                  wait=0 returns error if board busy
  * @param qual Sets JPEG quality, (use 25-100, default 100).
  * @param unused (reserved for future use, value ignored)
  * @param hdev of 2226.
  * @return number of bytes read (image size) or 
  *    MID2226_SNAP_ERR_UNSPECIFIED -1
  *    MID2226_SNAP_ERR_BUFSIZE     -2  'Size' of buffer provided is too small for snapshot.
  *    MID2226_SNAP_ERR_BIST        -3  Built in self test failed. Hardware not ready.
  *    MID2226_SNAP_ERR_SYNCH       -4  Aquisition's synchronization loop timed out.
  *    MID2226_SNAP_ERR_CRITSEC     -5  Aquisition's critical section already locked, and user doesn't want to wait for it.
  *    MID2226_SNAP_ERR_PARTBUF     -6  Didn't recieve a full frame of data.
  *    MID2226_SNAP_ERR_DVBUF       -7  Failed to get current frame buffer from DV
  *    MID2226_SNAP_ERR_DVGRAB      -8  The DV frame grabber was not setup
  *    MID2226_SNAP_ERR_USERERR     -9  Error returned from user's merge function
  */
MID2226_API int __stdcall S2226_SnapshotToFile( char *szFilename, int filetype, int freezetime, int wait, int qual, int unused, HANDLE hdev);
 
MID2226_API int __stdcall S2226_SnapshotToFileW(const wchar_t *pFile, int filetype, int freezetime, int wait, int qual, int unused, HANDLE hdev);

///** Get snapshot to memory function.  Retrieves processed image into memory.
//  * Image is converted to RGB (1 byte per color) and then the fields are merged 
//  * using the algorithm set by S2226_SetMergeMethod().
//  * @param image IN/OUT pointer to retrieved snapshot.  
//  * @param size is the size of the image buffer.
//  * @param freezetime is the time, in milli-sec to freeze the image on the monitor. (a minimum value greater than the video frame rate is required)
//  * @param wait wait =1 will wait if operations pending(board locked), 
//  *                  wait=0 returns error if board busy
//  * @param hdev of 2226.
//  * @return number of bytes read (image size) or 
//  *    MID2226_SNAP_ERR_UNSPECIFIED -1
//  *    MID2226_SNAP_ERR_BUFSIZE     -2  'Size' of buffer provided is too small for snapshot.
//  *    MID2226_SNAP_ERR_BIST        -3  Built in self test failed. Hardware not ready.
//  *    MID2226_SNAP_ERR_SYNCH       -4  Aquisition's synchronization loop timed out.
//  *    MID2226_SNAP_ERR_CRITSEC     -5  Aquisition's critical section already locked, and user doesn't want to wait for it.
//  *    MID2226_SNAP_ERR_PARTBUF     -6  Didn't recieve a full frame of data.
//  *    MID2226_SNAP_ERR_DVBUF       -7  Failed to get current frame buffer from DV
//  *    MID2226_SNAP_ERR_DVGRAB      -8  The DV frame grabber was not setup
//  *    MID2226_SNAP_ERR_USERERR     -9  Error returned from user's merge function
//  */
MID2226_API int __stdcall S2226_SnapshotToMem( BYTE *image, int size, int freezetime,int wait, HANDLE hdev);

/** Raw image structure
 *  @param f0 pointer to field 0
 *  @param size0 size of field 0
 *  @param f1 pointer to field 1 or null if not present
 *  @param size1 size of field 1
 */
typedef struct 
{
    BYTE *f0;
    int  size0;
    BYTE *f1;
    int  size1;
} image_raw_t;

///** Get snapshot data from hardware only.  This function does no processing on the image, it just
//  * retrieves a pointer to the raw fields(field 1 will be null for 720p format) in YCrCb format
//  * @param image is a pointer to pre-allocated space for image
//  * @param size is the size of image array above
//  * @param pImage is a pointer to the returned raw image fields
//  * @param freezetime is the time, in milli-sec to freeze the image on the monitor. (a minimum value greater than the video frame rate is required)
//  * @param wait wait=1 will wait if operations pending(board locked), 
//  *             wait=0 returns error if board busy
//  * @param hdev of 2226.
//  * @return number of bytes read (image size) or 
//  *    MID2226_SNAP_ERR_UNSPECIFIED -1
//  *    MID2226_SNAP_ERR_BUFSIZE     -2  'Size' of buffer provided is too small for snapshot.
//  *    MID2226_SNAP_ERR_BIST        -3  Built in self test failed. Hardware not ready.
//  *    MID2226_SNAP_ERR_SYNCH       -4  Aquisition's synchronization loop timed out.
//  *    MID2226_SNAP_ERR_CRITSEC     -5  Aquisition's critical section already locked, and user doesn't want to wait for it.
//  *    MID2226_SNAP_ERR_PARTBUF     -6  Didn't recieve a full frame of data.
//  */
MID2226_API int __stdcall S2226_SnapshotRaw( BYTE *image, int size, image_raw_t *pImage, int freezetime, int wait, HANDLE hdev);

MID2226_API int __stdcall S2226_GetFirmwareVersions(unsigned int *fpga, unsigned int *usb, unsigned int *cpu, unsigned int *boardid, HANDLE hdev);

// Get software SDK versions
// num: number of versions to retrieve (currently 3 are available)
// vers: pointer to array of versions
#define IDX_VERSION_DRIVER_SYS 0 // main driver (version loaded in memory)
#define IDX_VERSION_DRIVER_PROXY 1 // driver proxy
#define IDX_VERSION_DLL 2 // DLL
MID2226_API int __stdcall S2226_GetSDKVersions(unsigned int num, unsigned int *vers, HANDLE hdev);


// 2226 device messages (unplug and and stream done) related to Windows DirectShow
MID2226_API int __stdcall S2226_SetNotify(HWND hNotifyApp, UINT mNotifyMessage, HANDLE hdev);
// If message received, tests if device was removed(unplug)
MID2226_API int __stdcall S2226_TestDeviceRemoval(HANDLE hdev);
// If message received, tests if decode was finished in the driver.
// This is NOT, however, when decode stops on the codec chip.
// It is only when the driver is finished with the data.
//  A future update will correct this limitation.
// Because the data is compressed and there are buffers on the board,
//  TestDecodeDone may be early by a substantial amount.
MID2226_API int __stdcall S2226_TestDecodeDone(HANDLE hdev);


// call on WM_DISPLAYCHANGE message
MID2226_API int __stdcall S2226_DisplayChange(HDC hdc, HANDLE hdev);


MID2226_API int __stdcall S2226_Repaint(HDC hdc, HANDLE hdev);

MID2226_API int __stdcall S2226_GetSN(SN2226 *sn, HANDLE hdev);


// Important Note: As of 9/28/2012, Rev B boards with FPGA firmware 45 do not require this setting
//                        Rev B boards with this firmware can perform raw preview while
//                        scaling the output.
//
// type = 1, turns scaled output on (HD streams will be scaled down to the composite outputs)
//          (Note: This feature disables raw preview)
// type = 0, turns scaled output off (Re-enables raw preview feature)
MID2226_API int __stdcall S2226_SetOutput(int type, HANDLE hdev);


// Under normal circumstances, there will not be any error messages. 
// The messages below are primarily for internal Sensoray use (debug, manufacturing),
// Notification of internal device error.
// If Error message with wparam 0x5151 is received, the stream has stopped.
MID2226_API int __stdcall S2226_SetNotifyError(HWND hNotifyApp, UINT mNotifyMessage, HANDLE hdev);
// only use if one receives an error notification message.  If unsure, do not use.
// num: number of registers (currently should be 6)
// reg: pointer to array of registers
MID2226_API int __stdcall S2226_GetErrorRegisters(int num, UINT16 *reg, HANDLE hdev);



// start decode from memory.  Allows decoding from memory instead of from file.
// Send new data to the board with the function S2226_SendData
MID2226_API int __stdcall S2226_StartDecodeMem(MID2226_DECODE_TYPE dt, HANDLE hdev); 

// S2226_SendData
MID2226_API int __stdcall S2226_SendData(unsigned char *data, int len, HANDLE hdev);

// Select Source feeding the MPEG audio to be encoded
MID2226_API int __stdcall S2226_SetAudioMuxMpegIn(MID2226_MUX_MPEG_IN_TYPE source, HANDLE hdev);

// Select Source feeding the audio Line out
MID2226_API int __stdcall S2226_SetAudioMuxLineOut(MID2226_MUX_LINE_OUT_TYPE source, HANDLE hdev);

// Select Source feeding the SDI-Out embedded audio encoder
MID2226_API int __stdcall S2226_SetAudioMuxSdiOut(MID2226_MUX_SDI_OUT_TYPE source, HANDLE hdev);

/** Set Audio Meter Hold time
 * 0 = no hold (Update every 1 ms)  
 * 1 = Hold hold highest value 0.5 sec
 * 2 = Hold hold highest value 1.0 sec
 * 3 = Hold hold highest value 1.5 sec
 * 4 = Hold hold highest value 2.0 sec
 * 5 = Hold hold highest value 2.5 sec
 * 6 = Hold hold highest value 3.0 sec
 * 7 = hold forever 
 */
MID2226_API int __stdcall S2226_SetAudioMtrHoldTime(int val, HANDLE handle);

/** Get Audio Meter Hold time
 */
MID2226_API int __stdcall S2226_GetAudioMtrHoldTime(int *val, HANDLE handle);

/** Set Audio Meter Hold Release
 * 1 = Force release of 'Held' output
 * 0 = Allow holding of highest db value per the set hold time
 */
MID2226_API int __stdcall S2226_SetAudioMtrHoldRelease(int val, HANDLE handle);

/** Get Audio Meter Hold Release value
 */
MID2226_API int __stdcall S2226_GetAudioMtrHoldRelease(int *val, HANDLE handle);

/** Set Audio Meter Test
 * 0 = No Test.        
 * 1 = Force to zero
 * 2 = Force to clip
 * 3 = Force to -6dB
 */
MID2226_API int __stdcall S2226_SetAudioMtrTest(int val, HANDLE handle);

/** Get Audio Meter Hold Test
 */
MID2226_API int __stdcall S2226_GetAudioMtrTest(int *val, HANDLE handle);

/** Set Audio Meter Channel Select. Determines which Audio
 * channel to send to the meter/moniter.
 * 0 = Channel 0 = Line-in 
 * 1 = Channel 1 = Mpeg-out
 * 2 = Channel 2 = SDI-in  
 * 3 = Channel 3           
 */
MID2226_API int __stdcall S2226_SetAudioMtrChnSel(int val, HANDLE handle);

/** Get Audio Meter Channel Select
 */
MID2226_API int __stdcall S2226_GetAudioMtrChnSel(int *val, HANDLE handle);

/** Get Audio Meter Peak and Decayed Level 
 * 23-bits, unsigned binary 0=min volume
 */
MID2226_API int __stdcall S2226_GetAudioMtrLevel(int *audl, int *audr, HANDLE handle);

/** Get Audio Meter Peak and Decayed Level in dB
 * 11-bits, unsigned binary 0=max volume 
 * 2048 steps, -0.1 db each
 */
MID2226_API int __stdcall S2226_GetAudioMtr_dB(int *db_l, int *db_r, HANDLE handle);

/** Hold value and Clip detect
 * hld_l  : 11-bits, unsigned binary 0=max volume. 2048 steps, -0.1 db each
 * hld_r  : 11-bits, unsigned binary 0=max volume. 2048 steps, -0.1 db each
 * clip_l : 1 indicates (0x7FFFFF or 0x800000 detected)
 * clip_r : 1 indicates (0x7FFFFF or 0x800000 detected)
 */
MID2226_API int __stdcall S2226_GetAudioMtrHold(int *hld_l, int *hld_r, int *clip_l, int *clip_r, HANDLE handle);

// checks the input to see if it was disconnected.  
MID2226_API int __stdcall S2226_CheckInput(int *ilst, HANDLE handle);

/** Set LED's
 * Msb 8-15 are masks for setting and clearing Led 0-7
 *   i.e. Write 0x0101 to set   LED_0
 *        Write 0x0200 to clear LED_1
 *        Write 0x040F to set   LED_2 (Led's 0,1,3 ignored)
 */
MID2226_API int __stdcall S2226_SetLeds(int val, HANDLE handle);

/** Get Led settings
 */
MID2226_API int __stdcall S2226_GetLeds(int *val, HANDLE handle);

/** Set General Purpose IO
 *    TEST[0] = J1-4   J1-1 = TEST[1]
 *    TEST[1] = J1-1   J1-2 = VCC_3.3
 *    TEST[2] = J1-3   J1-3 = TEST[2]
 *    TEST[3] = J1-5   J1-4 = TEST[0]
 *    VCC_3.3 = J1-2   J1-5 = TEST[3]
 *    GND     = J1-6   J1-6 = GND    
 *
 *    gpo[7-0] Set by val[15-8]
 *    gpo[7-4] Controls whether TEST[3-0] is an input(0) or output(1)
 *    gpo[3-0] Drives TEST[3-0] if set as an output
 * i.e.
 *    Write 0xC800... gpo = 1100 1000 binary
 *      TEST[3] is driven to 1
 *      TEST[2] is driven to 0
 *      TEST[1-0] are inputs
 */
MID2226_API int __stdcall S2226_SetGPIO(int val, HANDLE handle);

/** Get General Purpose IO
 *   Bits 15-8 = gpo[7-0] = Bits 15-8 written via S2226_SetGPIO()
 *   Bits  7-4 = zero
 *   Bits  3-0 = Input from TEST[3-0] if configured as input
 */
MID2226_API int __stdcall S2226_GetGPIO(int *val, HANDLE handle);

/**
 *  To be used with S2226_RegisterCallbackRaw
*/
MID2226_API int __stdcall S2226_StartRawPreviewCallback(HANDLE handle);


// helper functions not associated with the 2226 hardware
MID2226_API int __stdcall S2226_save_rgb(const char *szFilename, const unsigned char *rgb, int width, int height, int type);
MID2226_API int __stdcall S2226_save_rgbW(const wchar_t *szFilename, const unsigned char *rgb, int width, int height, int type);
MID2226_API int __stdcall S2226_yuy2_to_rgb(const unsigned char *yuy2, unsigned char *rgb, int width, int height);
MID2226_API int __stdcall S2226_uyvy_to_rgb(const unsigned char *uyvy, unsigned char *rgb, int width, int height);

#endif
